import * as process from 'process';
import { moduleExists } from '@sweet-milktea/utils';
import defaultPlugins from './utils/defaultPlugins.js';
import presetEnv from './utils/presetEnv.js';
import transformRuntime from './utils/transformRuntime.js';
const isDevelopment = process.env.NODE_ENV === 'development';
function babelPresetSweet(api, options = {}, dirname) {
    const { env, typescript, react, polyfill } = options;
    const { nodeEnv, ecmascript, targets: customTargets, debug, modules, useBuiltIns } = env !== null && env !== void 0 ? env : {}, { use: useTypescript, isReact = true } = typescript !== null && typescript !== void 0 ? typescript : {}, { use: useReact = true, runtime, development } = react !== null && react !== void 0 ? react : {};
    const envModules = modules !== null && modules !== void 0 ? modules : false; // @babel/preset-env的模块类型
    // 编译目标
    let babelBuildTargets;
    if (customTargets) {
        babelBuildTargets = customTargets;
    }
    else {
        if (ecmascript) {
            babelBuildTargets = {
                browsers: nodeEnv ? ['node 14'] : ['last 3 Chrome versions']
            };
        }
        else {
            babelBuildTargets = {
                browsers: nodeEnv ? ['node 10'] : [
                    'last 10 versions',
                    'last 10 Chrome versions',
                    'last 1 year',
                    'IE 11'
                ]
            };
        }
    }
    const presets = [];
    const plugins = defaultPlugins.concat([
        transformRuntime({
            ecmascript,
            nodeEnv,
            polyfill
        })
    ]);
    // 添加@babel/preset-env
    presets.push(presetEnv({
        babelBuildTargets,
        useBuiltIns,
        debug,
        envModules,
        polyfill
    }));
    // 添加@babel/preset-react
    if (useReact) {
        presets.push([
            '@babel/preset-react',
            {
                runtime: runtime !== null && runtime !== void 0 ? runtime : (moduleExists('react/jsx-runtime') ? 'automatic' : 'classic'),
                development: development !== null && development !== void 0 ? development : isDevelopment
            }
        ]);
    }
    // 添加@babel/preset-typescript
    if (useTypescript) {
        presets.push([
            '@babel/preset-typescript',
            {
                isTSX: true,
                jsxPragma: isReact ? 'React' : 'Preserve',
                allExtensions: true,
                allowNamespaces: true
            }
        ]);
    }
    // 添加babel-plugin-polyfill-{name}相关插件
    if (polyfill) {
        plugins.push([
            'babel-plugin-polyfill-corejs3',
            {
                method: 'usage-pure',
                targets: babelBuildTargets
            }
        ], [
            'babel-plugin-polyfill-es-shims',
            {
                method: 'usage-pure',
                targets: babelBuildTargets
            }
        ]);
    }
    return { presets, plugins };
}
export default babelPresetSweet;
